#!/usr/bin/python3

"""Colector de palabras.

Cómo usar:

    >>> import pals
    >>> pals.get_palabra(el_largo_de_la_palabra_que_quiero)
    bleh

"""

import json
import random
import re
import unicodedata
import sys
from collections import Counter


def norm(txt):
    return unicodedata.normalize('NFKD', txt).encode('ASCII', 'ignore').lower().decode('ascii')


def expand(base, variation):
    # simple case of word ending in 'o' and variation to 'a'
    if base[-1] == 'o' and variation == 'a':
        transformed = base[:-1] + 'a'
        return base, transformed

    # triz! base must end in 'or'
    if base[-2:] == 'or' and variation == 'triz':
        transformed = base[:-2] + 'triz'
        return base, transformed

    # ína! base must end in 'e'
    if base[-1] == 'e' and variation == 'ína':
        transformed = base[:-1] + 'ína'
        return base, transformed

    # if base ends in "ey", variation must end in "eina", and it's full
    if base[-2:] == 'ey' and variation[-4:] == 'eina':
        return base, variation

    # traceback the base to see the splitting point, to complete with the variation;
    # this is normally one letter, unless 'll' case
    if variation.startswith('ll') and 'll' in base:
        split_letter = 'll'
    else:
        split_letter = variation[0]
    try:
        split_pos = base.rindex(split_letter)
    except ValueError:
        pass
    else:
        # similar length ending
        transformed = base[:split_pos] + variation
        return base, transformed


def _get_words():

    with open("rae.json", "rt", encoding="utf8") as fh:
        data = json.load(fh)

    raw_words = sorted({w.strip() for w in data.keys()})
    words = []
    for word in raw_words:
        parts = [x.strip() for x in word.split(",")]

        if len(parts) == 1:
            # simple word!
            words.extend(parts)
            continue

        if parts[0].endswith('-'):
            # prefix/postfix, e.g.: 'fono-, ‒́fono, na'
            continue

        base, variation = parts
        result = expand(base, variation)
        if result is not None:
            words.extend(result)

    words = sorted(map(norm, words))
    return words


def get_palabra(largo):
    """Devuelve una palabra al azar de ese largo."""
    words = [w for w in _get_words() if len(w) == largo]
    return random.choice(words)


def find(restric, chars):
    conteoorig = Counter(chars)
    regex = '^' + restric + '$'

    words = _get_words()
    results = set()
    for word in words:
        if re.match(regex, word):
            conteomatch = Counter(word)
            for l, q in conteomatch.items():
                if q > conteoorig.get(l, 0):
                    break
            else:
                results.add(word)
    return sorted(results)


if __name__ == '__main__':
    for r in find(sys.argv[2], sys.argv[1]):
        print(r)
